/* vi: set ts=4 sw=4: */
/* -*- mode: c; c-basic-offset: 4; tab-width: 4 -*- */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <limits.h>
#include "augen.h"

#define PI		3.1415926535897931
#define TWO_PI	6.2831853071795862

#define SAMPLE_MAX_FLT	32768.0

#define NUM_SAMPLES(ms)	((ms) * srate / 1000)

static int srate = 44100;
static int channels = 2;

/* creates a buffer that can hold so many milliseconds */
struct ag_buffer *ag_buffer_create(long msec) {
	struct ag_buffer *buf;

	if(!(buf = malloc(sizeof *buf))) {
		return 0;
	}
	buf->size = ag_buffer_size(msec, sizeof(wsample_t));
	if(!(buf->data = malloc(buf->size))) {
		free(buf);
		return 0;
	}
	buf->samples = NUM_SAMPLES(msec);

	ag_buffer_clear(buf);

	return buf;
}

/* frees the buffer */
void ag_buffer_free(struct ag_buffer *buf) {
	if(buf) {
		free(buf->data);
	}
	free(buf);
}

void ag_buffer_clip(sample_t *dest, struct ag_buffer *buf) {
	int i;
	for(i=0; i<buf->samples; i++) {
		dest[i] = (sample_t)buf->data[i];
	}
}

void ag_buffer_norm(sample_t *dest, struct ag_buffer *buf) {
	int i;
	wsample_t min = INT32_MAX, max = INT32_MIN;
	float scale;
	
	for(i=0; i<buf->samples; i++) {
		wsample_t s = buf->data[i];
		if(s > max) max = s;
		if(s < min) min = s;
	}
	if(min > max) max = min;
	scale = SAMPLE_MAX_FLT / (float)max;

	for(i=0; i<buf->samples; i++) {
		dest[i] = (sample_t)((float)buf->data[i] * scale);
	}
}

/* calculates the buffer size needed for an arbitrary number of milliseconds */
size_t ag_buffer_size(long msec, size_t sample_size) {
	return NUM_SAMPLES(msec) * channels * sample_size;
}

/* generate a sound wave */
void ag_gen_wave(struct ag_buffer *buf, long msec, float freq, float ampl, int type) {
	int i, samples = buf->samples;
	
	for(i=0; i<samples; i+=2) {
		wsample_t s;
		float t = (float)i / (float)srate;
		
		switch(type) {
		case AG_WAVE_SIN:
			s = (wsample_t)(ampl * sin(TWO_PI * t * freq) * SAMPLE_MAX_FLT);
			break;

		case AG_WAVE_SAW:
			{
				float x = t * freq / 2.0;
				s = (wsample_t)(ampl * (x - floor(x)) * SAMPLE_MAX_FLT);
			}
			break;
		}
		buf->data[i] = buf->data[i + 1] = s;
	}
}

void ag_buffer_clear(struct ag_buffer *buf) {
	memset(buf->data, 0, buf->size);
}

void ag_buffer_add(struct ag_buffer *dest, struct ag_buffer *b1, struct ag_buffer *b2) {
	int i;
	for(i=0; i<dest->samples; i++) {
		dest->data[i] = b1->data[i] + b2->data[i];
	}
}

int ag_buffer_cat(struct ag_buffer *dest, struct ag_buffer *src) {
	wsample_t *tmp = realloc(dest->data, dest->size + src->size);
	if(tmp) {
		dest->data = tmp;
		memcpy((char*)dest->data + dest->size, src->data, src->size);
		dest->size += src->size;
		return 0;
	}
	return -1;
}
